/*
	Sudoku generator

	(C) Copyright 2005 by Mogens Kjaer, mk@lemo.dk

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>

extern int nfilled,newnfilled,bestndigits;
extern int symmetrical;

extern int displayboard[9][9];

extern int setboard[9][9];
extern int setboardsave[9][9];
extern int beforeboard[9][9];
extern int board[9][9];
extern int nmove,nsol,ntest,running,solved;

static int randomize=1;

/*
	The following random number generator
	code is taken from glibc, containing
	the following copyright:

*/


/*
   Copyright (C) 1995, 2005 Free Software Foundation

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */

/*
   Copyright (C) 1983 Regents of the University of California.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

   1. Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
   2. Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
   4. Neither the name of the University nor the names of its contributors
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
   SUCH DAMAGE.*/

/*
 * This is derived from the Berkeley source:
 *	@(#)random.c	5.5 (Berkeley) 7/6/88
 * It was reworked for the GNU C Library by Roland McGrath.
 * Rewritten to be reentrant by Ulrich Drepper, 1995
 */

struct local_random_data
  {
    int *fptr;		/* Front pointer.  */
    int *rptr;		/* Rear pointer.  */
    int *state;		/* Array of state values.  */
    int rand_type;		/* Type of random number generator.  */
    int rand_deg;		/* Degree of random number generator.  */
    int rand_sep;		/* Distance between front and rear.  */
    int *end_ptr;		/* Pointer behind state table.  */
  };

static int randtbl[32] =
  {
    3,

    -1726662223, 379960547, 1735697613, 1040273694, 1313901226,
    1627687941, -179304937, -2073333483, 1780058412, -1989503057,
    -615974602, 344556628, 939512070, -1249116260, 1507946756,
    -812545463, 154635395, 1388815473, -1926676823, 525320961,
    -1009028674, 968117788, -123449607, 1284210865, 435012392,
    -2017506339, -911064859, -370259173, 1132637927, 1398500161,
    -205601318,
  };


static struct local_random_data unsafe_state =
  {
    .fptr = &randtbl[4],
    .rptr = &randtbl[1],
    .state = &randtbl[1],
    .rand_type = 3,
    .rand_deg = 31,
    .rand_sep = 3,
    .end_ptr = &randtbl[sizeof (randtbl) / sizeof (randtbl[0])]
};

static int local_random()
{
  int *state;
  int result;

  int *fptr = unsafe_state.fptr;
  int *rptr = unsafe_state.rptr;
  int *end_ptr = unsafe_state.end_ptr;
  int val;

  state = unsafe_state.state;

  val = *fptr += *rptr;
  /* Chucking least random bit.  */
  result = (val >> 1) & 0x7fffffff;
  ++fptr;
  if (fptr >= end_ptr)
    {
      fptr = state;
      ++rptr;
    }
  else
    {
      ++rptr;
      if (rptr >= end_ptr)
	rptr = state;
    }
  unsafe_state.fptr = fptr;
  unsafe_state.rptr = rptr;
  return result;
}

int local_srandom(unsigned int seed)
{
  int type;
  int *state;
  long int i;
  long int word;
  int *dst;
  int kc;

  type = unsafe_state.rand_type;

  state = unsafe_state.state;
  if (seed == 0)
    seed = 1;
  state[0] = seed;

  dst = state;
  word = seed;
  kc = unsafe_state.rand_deg;
  for (i = 1; i < kc; ++i)
    {
      long int hi = word / 127773;
      long int lo = word % 127773;
      word = 16807 * lo - 2836 * hi;
      if (word < 0)
	word += 2147483647;
      *++dst = word;
    }

  unsafe_state.fptr = &state[unsafe_state.rand_sep];
  unsafe_state.rptr = &state[0];
  kc *= 10;
  while (--kc >= 0)
    {
      (void) local_random();
    }

 done:
  return 0;

 fail:
  return -1;
}

static void permute(int n, int *values)
{
  int i,j,tmp;

  for(i=0;i<(n-1);i++)
  {
    j=local_random()%(n-i)+i;
    tmp=values[i];
    values[i]=values[j];
    values[j]=tmp;
  }
}

static int check(int r, int c, int n)
{
  int i,j;
  for(i=0;i<9;i++)
  { if(setboard[i][c]==n) return 0; if(setboard[r][i]==n) return 0; }
  for(i=0;i<3;i++) for(j=0;j<3;j++)
  { if(setboard[r/3*3+i][c/3*3+j]==n) return 0; }
  return 1;
}

static void move()
{
  int i,j,i1,j1,p,best,bestp,n,r,c,empty;
  int indices[10];
  int nindices;
  nmove++;
  r = -1;
  c = -1;
  best = 999;
  empty = 0;

  for(i=0; i<9; i++)
  {
    for(j=0; j<9; j++)
    {
      if(board[i][j])
      {
	p=0;
	empty++;
      }
      else
      {
	p=1022;
	for(i1=0; i1<9; i1++)
	{
	  p &= ~(1<<board[i][i1]);
	  p &= ~(1<<board[i1][j]);
	}
	for(i1=0;i1<3;i1++)
	for(j1=0;j1<3;j1++)
	  p &= ~(1<<board[i/3*3+i1][j/3*3+j1]);
	/* calculate n as the number of bits set
	   to 1 in p: */
	n = p;
	n = (0x5555 & n) + (0x5555 & (n>> 1));
        n = (0x3333 & n) + (0x3333 & (n>> 2));
	n = (0x0f0f & n) + (0x0f0f & (n>> 4));
	n = (0x00ff & n) + (0x00ff & (n>> 8));
	if(n<best)
	{
	  best=n;
	  bestp=p;
	  r = i;
	  c = j;
	  /* if n is zero, we've reach a dead end */
	  if(n==0) return;
	}
      }
    }
  }

  /* if r is -1, all cells are filled, and we're done */
  if(r== -1)
  {
    nsol++;
    return;
  }
  if(randomize && best>1)
  {
    nindices=0;
    for(i=1;bestp;i++)
    {
      bestp >>= 1;
      if(1 & bestp)
      {
        indices[nindices++]=i;
      }
    }
    permute(nindices, indices);
    for(i=0;i<nindices;i++)
    {
      board[r][c]=indices[i];
      move();
      if(nsol>1) return;
      board[r][c]=0;
    }
  }
  else
  {
    for(i=1;bestp;i++)
    {
      bestp >>= 1;
      if(1 & bestp)
      {
	board[r][c]=i;
	move();
	if(nsol>1) return;
	board[r][c]=0;
      }
    }
  }
}

void sudoku_run()
{
  int i,j,i1,j1,ndigits;
  int i2,j2,k2;
  int walktype;
  char windowtitle[80];

  solved=0;

  ntest++;

  for(i=0; i<9; i++)
  for(j=0; j<9; j++)
  board[i][j] = 0;

  randomize=1;

  nsol=0;
  move();
  if(nsol==0)
  {
    fprintf(stderr, "Hm, can't happen.\n");
    exit(0);
  }

  for(i=0; i<9; i++)
  for(j=0; j<9; j++)
  setboardsave[i][j] = board[i][j];

  randomize=0;

  if(symmetrical)
  {
    for(walktype=0;walktype<4;walktype++)
    {
      for(i1=0; i1<9; i1++)
      for(j1=0; j1<9; j1++)
      setboard[i1][j1] = setboardsave[i1][j1];
      ndigits=81;
      for(i2=0;i2<5;i2++)
      for(j2=0;j2<5;j2++)
      {
	i=i2;
	j=j2;
	if(walktype&1) i=4-i;
	if(walktype&2) j=4-j;
	for(i1=0; i1<9; i1++)
	for(j1=0; j1<9; j1++)
	board[i1][j1] = setboard[i1][j1];
	board[4+i][4+j] = 0;
	board[4+i][4-j] = 0;
	board[4-i][4+j] = 0;
	board[4-i][4-j] = 0;
	nsol=0;
	move();
	if(nsol==1)
	{
	  setboard[4+i][4+j] = 0;
	  setboard[4+i][4-j] = 0;
	  setboard[4-i][4+j] = 0;
	  setboard[4-i][4-j] = 0;
	  if((i==0)&&(j==0))
	  {
	    ndigits -= 1;
	  }
	  else if((i==0)||(j==0))
	  {
	    ndigits -= 2;
	  }
	  else
	  {
	    ndigits -= 4;
	  }
	}
      }
      if(ndigits<=bestndigits)
      {
	for(i=0; i<9; i++)
	for(j=0; j<9; j++)
	displayboard[i][j] = setboard[i][j];
	bestndigits=ndigits;
	sprintf(windowtitle, "Sudoku Generator. %d cells in %d iteration%s", ndigits, ntest, ntest==1?"":"s");
	settitle(windowtitle);
	main_update();
	if(bestndigits<=nfilled)
	{
	  running=0;
	  setcreatemenutext("Create");
	  return;
	}
      }
    }
  }
  else
  {
    for(walktype=0;walktype<16;walktype++)
    {
      ndigits=81;

      for(i1=0; i1<9; i1++)
      for(j1=0; j1<9; j1++)
      setboard[i1][j1] = setboardsave[i1][j1];

      for(i2=0; i2<9; i2++)
      for(j2=0; j2<9; j2++)
      {
	i=i2;
	j=j2;
	if(walktype&1) i=8-i;
	if(walktype&2) j=8-j;
	if(walktype&4) i=(i+4)%9;
	if(walktype&8) j=(j+4)%9;
	for(i1=0; i1<9; i1++)
	for(j1=0; j1<9; j1++)
	board[i1][j1] = setboard[i1][j1];

	board[i][j] = 0;
	nsol=0;
	move();
	if(nsol==1)
	{
	  setboard[i][j] = 0;
	  ndigits--;
	}
      }
      if(ndigits<=bestndigits)
      {
	for(i=0; i<9; i++)
	for(j=0; j<9; j++)
	displayboard[i][j] = setboard[i][j];
	bestndigits=ndigits;
	sprintf(windowtitle, "Sudoku Generator. %d cells in %d iteration%s", ndigits, ntest, ntest==1?"":"s");
	settitle(windowtitle);
	main_update();
	if(bestndigits<=nfilled)
	{
	  running=0;
	  setcreatemenutext("Create");
	  return;
	}
      }
    }
  }
  /*
  sprintf(windowtitle, "Sudoku Generator. Iteration %d, filled: %d, best: %d", ntest, ndigits, bestndigits);
  settitle(windowtitle);
  */
}

void unsolve_sudoku()
{
  int i,j;
  for(i=0; i<9; i++)
  for(j=0; j<9; j++)
  displayboard[i][j] = beforeboard[i][j];
  main_update();
  solved=0;
  setsolvemenutext("Solve");
}

void solve_sudoku()
{
  int i,j;
  for(i=0; i<9; i++)
  for(j=0; j<9; j++)
  {
    board[i][j] = displayboard[i][j];
    beforeboard[i][j] = displayboard[i][j];
  }
  randomize=0;
  nsol=1;
  move();
  if(nsol>0)
  {
    for(i=0; i<9; i++)
    for(j=0; j<9; j++)
    displayboard[i][j] = beforeboard[i][j]?board[i][j]:-board[i][j];
    main_update();
    solved=1;
    setsolvemenutext("Unsolve");
  }
}

